/*
 *	bull_board.c 
 *	
 *	- generic bulletin board object
 *	- modified by Truilkan to allow read/write permissions
 *	  and assignment of projects
 *	- rewritten by Truilkan to use a mapping for the messages
 *	- Zak 930119:	added 'followup', 'mailnote'
 *   	- Shemp 930224	added maker stuff
 *	- Zak, 930225	made it range from 1..x instead of 0..x-1
 *	- Shemp, 930226	made the note removed message reflect the
 *			actually note removed instead of note-1
 *	- Shemp, 930228	added function to enable an admin to "lock" a
 *			note so that it doesn't get removed with the
 *			automatic removal of aged notes.
 *	- Zak, 930309	changed to new output format. Optimised a bit too.
 *			Also checks before mkdir({save,attic}_dir).
 *			Store all `set'able things in temp vars, so they're
 *			not lost during the restore_object() in set_save_file
 *	- Zak, 930314	enhanced locking/unlocking of notes, won't archive
 *			locked notes.
 *	- Zak, 930320	only messages[] is non-static, num_messages
 *			is calculated at each restore_object()
 *	- Zak, 930321	set_default_long_message() to update yer
 *			bboard's default long.text.
 *
 *	- Watcher, 930322  Switched name recognition to use link to prevent
			   impersonation with monster bodies
 */

#include <move.h>
#include <mudlib.h>
#include <daemons.h>
#include <mailer.h>

inherit OBJECT;

#define START_SAVE	30
#define STOP_SAVE	10

#define QNAME		link_data("name")

// messages mapping contains "title", "poster", "time", "doer", "body", "locked"
// don't maps make great substitutes for structures? :)

mapping	*messages;

static int	num_messages;
static string	attic_dir, save_dir;
static mapping	title;
static string	file, location, maker;
static int	max_messages, orig_number, carryover;


string query_info(int i, string field)
{
    if (i < num_messages && i >= 0)
	return messages[i][field];
    else
	return "";
}

int query_prevent_shadow()
{
    return 1;
}

// these valid_* functions are designed to be masked by the one inheriting

int valid_note_read(int i)
{
    return 1;
}

int valid_post()
{
    return 1;
}

int valid_assign()
{
    return 0;
}

int valid_remove(int i)
{
    string poster;

    if ( messages[i]["locked"] != geteuid(this_player()) &&
         !member_group(geteuid(this_player()),"admin") &&
	 (string)this_player()->link_data("cap_name") != messages[i]["poster"]
	 && (string) this_player()->QNAME != maker )
    {
	notify_fail("You can only remove your own notes.\n");
	return 0;
    }
    return 1;
}

int valid_lock()
{
    return (member_group(geteuid(this_player()), "admin"));
}

void set_carryover(int t)
{
    carryover = t;
}

int query_carryover()
{
    return carryover;
}

int query_max_messages()
{
    return max_messages;
}

void set_max_messages(int num)
{ 
    if (num < 0)
	num = 0; 
    max_messages = num;
}

static void set_maker(string o) {
    maker = lower_case(o);
}

string query_maker() {
    return maker;
}

string get_time(int t)
{
    return extract(ctime(t), 0, 9);
}

string headers()
{
    string tmp;
    int i;

    if (!num_messages)
	return "";
    tmp = "";
    i = -1;
    while ( ++i < num_messages )
    {
	tmp += sprintf("%3d  %-12s (%s)%s %s%s\n",
		(i + 1),
		messages[i]["poster"],
		get_time(messages[i]["time"]),
                (messages[i]["locked"] ? "*" : " "),
		messages[i]["title"],
		(messages[i]["doer"]
			? "  <" + messages[i]["doer"] + ">" : "" ));
    }
    return tmp;
}

string query_long()
{
    return ( sprintf("%sThe %s has %s note%s:\n%s",
		(string) query("long.text"),
		(string) query("short.text"),
		(num_messages == 0 ? "no" : "" + num_messages),
		(num_messages == 1 ? "" : "s"), 
		headers()
	    ));
}

string query_short()
{
    return (sprintf("%s [%d note%s]", (string) query("short.text"), 
	    num_messages, (num_messages == 1 ? "" : "s") ));
}

int validate_num(int x)
{
    if (x < 1 || x > num_messages)
	return -1;
    return (x-1);
}

void set_save_dir(string s)
{
    save_dir = s + "/";
//    if (file_size(save_dir) != -2)
    if (file_size(save_dir) == -1)
	mkdir(save_dir);
}

string query_save_dir(string s)
{
    return save_dir;
}

void set_attic_dir(string s)
{
    attic_dir = s + "/";
//    if (file_size(attic_dir) != -2)
    if (file_size(attic_dir) == -1)
	mkdir(attic_dir);
}

string query_attic_dir()
{
    return attic_dir;
}

void set_location(string l)
{
    location = l;
    ob::move(l);
}

int move(mixed dest)
{
    return MOVE_NOT_ALLOWED;
}

void set_save_file(string s)
{
    int i, j, m;
    string *tmpa, temp_file;
    mapping *temp_messages;

    file = save_dir + s;
    seteuid(geteuid(previous_object()));
    restore_object(file);
    num_messages = sizeof(messages);
    if (num_messages<query_max_messages())
	return;
    tmpa = explode(file,"/");
    temp_file = attic_dir + tmpa[sizeof(tmpa) - 1] + "_" + time();
    temp_messages = messages;
    m = num_messages-query_carryover();
    messages = messages[0..m-1];
    save_object(temp_file);
    messages = temp_messages;
    for (j = i = 0; i < m; i++)
	if (messages[i]["locked"])
	    temp_messages[j++] = messages[i];
    messages = temp_messages[0..j-1] + messages[m..num_messages-1];
    num_messages = sizeof(messages);
    save_object(file);
}

void set_default_long_message()
{
    set("long.text", @LONGTEXT
    You can set up new notes with the command 'post [title]',
read a note with 'read #', remove an old note with 'remove #',
followup with 'followup # [title]', edit a note with 'edit #',
and mail a note to yourself with 'mailnote #'.
LONGTEXT
);
}

void create()
{
    seteuid(getuid(this_object()));
    set_max_messages(START_SAVE);
    set_attic_dir("/attic/boards");
    set("short", "@@query_short");
    set("long", "@@query_long");
    set("short.text", "A bulletin board");
    set_default_long_message();
    set("id",({"board","bulletin board"}));
    set_carryover(STOP_SAVE);
    messages = ({ });
    num_messages = 0;
    title = ([]);
}

void init()
{
    add_action("new_post","post");
    add_action("read", "read");
    add_action("remove_msg", "remove");
    add_action("edit_note","edit");
    add_action("assign","assign");
    add_action("followup_note", "followup");
    add_action("mail_note", "mailnote");
    add_action("lock_note", "mlock");
    add_action("unlock_note", "munlock");
    add_action("comment", "comment");
}

string unique_name()
{
    int pid;

    sscanf(file_name(this_player()),"/std/user#%d",pid);
    return "/tmp/bb_" + pid;
}

int assign(string str)
{
    int i;
    string who;

    if(this_player()->query("ghost")) {
    notify_fail("Your ghostly hands pass right through the board.\n");
    return 0; }
 
    if (!valid_assign())
	return 0;
    if (!str || sscanf(str,"%d %s",i,who) != 2)
    {
	notify_fail("Usage: assign <num> <name>\n");
	return 0;
    }
    i = validate_num(i);
    if (i != -1)
    {
	messages[i]["doer"] = capitalize(who);
	save_object(file);
	return 1;
    }
    else
	notify_fail("No note with that number.\n");
}

int lock_note(string str)
{
    int i;

    if(this_player()->query("ghost")) {
    notify_fail("Your ghostly hands pass right through the board.\n");
    return 0; }
 
    if (!str || sscanf(str, "%d", i) != 1)
    {
	notify_fail("Usage: mlock <num>\n");
	return 0;
    }
    i = validate_num(i);
    if (i == -1)
    {
	notify_fail("No note with that number.\n");
	return 0;
    }
    if (!valid_lock())
    {
	notify_fail("Permission denied - you may not lock that note.\n");
	return 0;
    }
    messages[i]["locked"] = geteuid(this_player());
    write("Note "+str+" locked.\n");
    return 1;
}

int unlock_note(string str)
{
    int i;

    if(this_player()->query("ghost")) {
    notify_fail("Your ghostly hands pass right through the board.\n");
    return 0; }
 
    if (!str || sscanf(str, "%d", i) != 1)
    {
	notify_fail("Usage: munlock <num>\n");
	return 0;
    }
    i = validate_num(i);
    if (i == -1)
    {
	notify_fail("No note with that number.\n");
	return 0;
    }
    if (!valid_lock())
    {
	notify_fail("Permission denied - you may not unlock that note.\n");
	return 0;
    }
    messages[i]["locked"] = 0;
    write("Note "+str+" unlocked.\n");
    return 1;
}

int comment(string str)
{
    int i;

    if(this_player()->query("ghost")) {
    notify_fail("Your ghostly hands pass right through the board.\n");
    return 0; }
 
    if (!str || sscanf(str, "%d", i) != 1)
    {
	notify_fail("Usage: comment <num>\n");
	return 0;
    }
    i = validate_num(i);
    if (i == -1)
    {
	notify_fail("No note with that number.\n");
	return 0;
    }
    write("This function does nothing at the moment.\n");
    return 1;
}

int new_post(string str)
{
    if(this_player()->query("ghost")) {
    notify_fail("Your ghostly hands pass right through the board.\n");
    return 0; }

    if (!valid_post())
	return 0;
    if (!str)
	str = "< no title >";
    if(title[this_player()])
	write("Warning! You are already marked as posting!\n");

    if (num_messages >= query_max_messages())
    {
	write("Too many messages.  You must remove some first.\n");
	return 1;
    }
    if (strlen(str) > 50)
    {
	write("Title is too long.\n");
	return 1;
    }
    title[this_player()] = str;
    write("Editing note.\n");
    this_player()->edit(unique_name(),"postit", this_object());
    return 1;
}

void abort()
{
    rm ((string)this_player()->query_edit_filename());
    map_delete(title,this_player());
}

varargs int postit(string mtitle)
{
    string filename;
    mapping msg;

    filename = (string)this_player()->query_edit_filename();
    msg = allocate_mapping(4);
    msg["title"] = mtitle ? mtitle : title[this_player()];
    msg["poster"] = capitalize((string)this_player()->QNAME);
    msg["time"] = time();
    msg["body"] = read_file(filename);
    rm(filename);
    this_player()->set_edit_filename("");
    if (!msg["body"])
	return 0;
    messages += ({ msg });
    write("Posted.\n");
    num_messages++;
    map_delete(title,this_player());
    save_object(file);
    return 1;
}

int read(string str)
{
    int i;
    string tmp;
   
    if (id(str)) {  write(query_long());  return 1;  }
 
    if (!str || sscanf(str, "%d", i) != 1)
    {
	notify_fail("Usage: read <num>\n");
	return 0;
    }
    i = validate_num(i);
    if (i == -1)
    {
	notify_fail("No note with that number\n");
	return 0;
    }
    if (this_object()->valid_note_read(i))
    {
	printf("Note %3d  %-12s (%s)  %s%s\n",
		(i + 1),
		messages[i]["poster"],
		get_time(messages[i]["time"]),
		messages[i]["title"],
		(messages[i]["doer"]
			? "  <" + messages[i]["doer"] + ">" : "" ));
	this_player()->more(({""}) + explode(messages[i]["body"],"\n"));
	return 1;
    }
    return 0;
}

int remove_msg(string str)
{
    string poster,tmp, *tmpmsg;
    int i,j;
   
    if(this_player()->query("ghost")) {
    notify_fail("Your ghostly hands pass right through the board.\n");
    return 0; }

    if (!str || sscanf(str, "%d", i) != 1)
    {
	notify_fail("Usage: remove <num>\n");
	return 0;
    }
    i = validate_num(i);
    if (i == -1)
    {
	notify_fail("No note with that number\n");
	return 0;
    }
    if (!valid_remove(i))
	return 0;
    messages = messages[0..(i-1)]+messages[(i+1) .. (sizeof(messages) - 1)];
    write ("Note "+(i+1)+" removed.\n");
    num_messages--;
    save_object(file);
    return 1;
}

int edit_note(string str)
{
    int i, tmp, j, m;
    string *lines, fl;
   
    if(this_player()->query("ghost")) {
    notify_fail("Your ghostly hands pass right through the board.\n");
    return 0; }

    if (!str || sscanf(str, "%d", i) != 1)
    {
	notify_fail("Usage: edit <num>\n");
	return 0;
    }
    i = validate_num(i);
    if (i == -1)
    {
	notify_fail("No note with that number\n");
	return 0;
    }
    if (messages[i]["locked"])
    {
	notify_fail("That message is locked\n");
	return 0;
    }
    if (!member_group(geteuid(this_player()), "admin") &&
	messages[i]["poster"] !=
		capitalize((string)this_player()->QNAME))
    {
	write("You may only edit your own messages.\n");
	return 1;
    }
    fl = unique_name();
    rm(fl);
    write_file(fl, messages[i]["body"]);
#if 0
    lines = explode(messages[i]["body"],"\n");
    m = sizeof(lines);
    for(j = 0; j < m; j++)
	write_file(fl,lines[j] + "\n");
#endif
    orig_number = i;
    write ("Editing : " + messages[i]["title"] + "\n");
    this_player()->edit(fl,"edit_stop_ed",this_object());
    return 1;
}

void edit_stop_ed()
{
    messages[orig_number]["body"] =
	read_file((string)this_player()->query_edit_filename());
    write ("Note "+(orig_number + 1)+" edited.\n");
    abort();
    save_object(file);
}

int followup_note(string str)
{
    string hed, *lines;
    int i;

    if(this_player()->query("ghost")) {
    notify_fail("Your ghostly hands pass right through the board.\n");
    return 0; }
 
    if (!valid_post())
	return 0;
    if (!str ||
	(sscanf(str, "%d %s", i, hed) != 2 &&
	 sscanf(str, "%d", i) != 1))
    {
	notify_fail("Usage: followup <num> [<title>]\n");
	return 0;
    }
    i = validate_num(i);
    if (i == -1)
    {
	notify_fail("No note with that number\n");
	return 0;
    }
    if (num_messages >= query_max_messages())
    {
	write("Too many messages.  You must remove some first.\n");
	return 1;
    }
    if(title[this_player()])
	write("Warning! You are already marked as posting!\n");
    if (!hed)
	hed = "Re: " + messages[i]["title"];
    if (strlen(hed) > 50)
    {
	write("Title is too long.\n");
	return 1;
    }
    title[this_player()] = hed;
    printf("Following up to note %d as '%s'\n", i +  1, hed);
    lines = explode(messages[i]["body"],"\n");
    hed = unique_name();
    rm(hed);
    write_file(hed, sprintf("On %s, %s writes:\n> %s\n",
		    get_time(messages[i]["time"]),
		    messages[i]["poster"],
		    implode(lines, "\n> ")));
//    write_file(hed, "On " + get_time(messages[i]["time"]) + ", " + 
//		    messages[i]["poster"] + " writes:\n" );
//    for(i = 0; i < sizeof(lines); i++)
//	write_file(hed,"> " + lines[i] + "\n");
    this_player()->edit(hed,"postit",this_object());
    return 1;
}

int mail_note(string str)
{
    int i;
    string myname, msg, *r;

    if(this_player()->query("ghost")) {
    notify_fail("Your ghostly hands pass right through the board.\n");
    return 0; }
 
    if (!str || sscanf(str, "%d", i) != 1)
    {
	notify_fail("Usage: mailnote <num>\n");
	return 0;
    }
    i = validate_num(i);
    if (i == -1)
    {
	notify_fail("No note with that number\n");
	return 0;
    }
    msg = sprintf("Title: %s\nFrom:  %s\nDate:  %s\n%s\n%s\n",
		    messages[i]["title"],
		    messages[i]["poster"],
		    get_time(messages[i]["time"]),
		    (messages[i]["doer"]
			? "Doer:  " + messages[i]["doer"] + "\n" : "" ),
		    messages[i]["body"]
		);

    myname = (string) this_player()->QNAME;
    r = (string *)MAILER_D->send_mail(
	    ({ myname }), ({}), myname,
	    "Note '" + messages[i]["title"] + "'", time(), msg );
    if (sizeof(r) != 1)
	write("Something went wrong with the mailing\n");
    else
    {
	write("Note '" + messages[i]["title"] + "' mailed to you.\n");
//	MAIL_SERVER->flush_mail_queue();
	MAILER_D->flush_files();
    }
    return 1;
}

void clean_up()
{
    return;
}
