/*
* bull_board.c 
*   - generic bulletin board object
*   - modified by Truilkan to allow read/write permissions
*   - and assignment of projects
*   - rewritten by Truilkan to use a mapping for the messages
*   - Mobydick added a valid_edit function.
*   - Zak added forwarding and saving to files.
*/

#include <move.h>
#include <mudlib.h>

inherit OBJECT ;

#define START_SAVE 30
#define STOP_SAVE  10

mapping *messages;
int num_messages;
static int max_messages;
string attic_dir, save_dir;

// messages mapping contains "title", "poster", "time", "doer"
// don't maps make great substitutes for structures? :)

static mapping title;
static string file;
static int orig_number, carryover;
static string location;

string query_info(int i, string field)
{
	if (i < num_messages)
		return messages[i][field];
	else
		return "";
}

int query_prevent_shadow()
{
   return 1;
}

/* these valid_* functions are designed to be masked by the one inheriting */

int valid_note_read(int i)
{
   return 1;
}

int valid_post()
{
   return 1;
}

int valid_assign()
{
   return 0;
}

int valid_remove(int i)
{
   string poster;

   if (!master()->query_member_group(geteuid(this_player()),"admin") &&
         (string)this_player()->query("cap_name") != query_info(i,"poster")) {
      notify_fail("You can only remove your own messages.\n");
      return 0;
   }
   return 1;
}

int valid_edit(int i)
{
   string poster;

   if (!master()->query_member_group(geteuid(this_player()),"admin") &&
         (string)this_player()->query("cap_name") != query_info(i,"poster")) {
      notify_fail("You can only edit your own messages.\n");
      return 0;
   }
   return 1;
}

void set_carryover(int t)
{
	carryover = t;
}

int query_carryover()
{
	return carryover;
}

int query_max_messages()
{
	return max_messages;
}

void set_max_messages(int num)
{ 
	if (num < 0)
		num = 0; 
	max_messages = num;
}

string get_time(int t)
{
   return extract(ctime(t), 0, 9);
}

string headers ()
{
	string tmp;
	int i;

   
	tmp = "";
	if (!messages) {
	  notify_fail("No messages.\n");
	  return 0;
	}
	for (i = 0; i < sizeof(messages); i++) {
		tmp += i + " [" + messages[i]["poster"] + "] ";
		tmp += messages[i]["title"] +
			" (" + get_time(messages[i]["time"]) + ")";
		if (messages[i]["doer"])
			tmp += " <" + messages[i]["doer"] + ">\n";
		else
			tmp += "\n";
   }
   return tmp;
}

string query_long()
{
   string tmp;
   tmp = "" + query("long.text");
   if (num_messages > 0) {  
      tmp += "The " + query("short.text") + " has " + num_messages;
      if (num_messages == 1)
         tmp = tmp + " note:\n";
      else
         tmp = tmp + " notes:\n";
   }
   else {
      return tmp + "The "+ query("short.text")+" has no messages.\n";
   }
   tmp = tmp + headers();
   return tmp;
}

string query_short()
{
   return (capitalize(query("short.text")) + " [" + num_messages + " messages]");
}

int assign(string str)
{
   int i;
   string who;

	if (!valid_assign())
		return 0;
	if (sscanf(str,"%d %s",i,who) != 2) {
		notify_fail("usage: assign # name\n");
		return 0;
	}
	if (i < num_messages) {
		messages[i]["doer"] = capitalize(who);
		save_object(file);
		return 1;
	}
	else {
		notify_fail("Not that many messages.\n");
	}
}

void init()
{
   add_action("new_post","post");
   add_action("read", "read");
   add_action("remove_msg", "remove");
   add_action("edit_note","edit");
   add_action("assign","assign");
   add_action ("followup_note", "followup") ;
// add_action ("save_note", "save") ;
}

void set_save_dir(string s)
{
	save_dir = s + "/";
}

string query_save_dir(string s)
{
	return save_dir;
}

void set_attic_dir(string s)
{
	attic_dir = s + "/";
}

string query_attic_dir()
{
	return attic_dir;
}

void create()
{
    set_max_messages(START_SAVE);
    set_attic_dir("/attic/boards");
    set("short", "@@query_short");
    set("long", "@@query_long");
    set("short.text", "a bulletin board");
    set("long.text",
          "You can set up new notes with the command 'post headline'.\n"+
	  "Read a note with 'read #', and remove an old note with\n" +
	  "'remove #'.  Edit a note with 'edit #'\n");
   set("id",({"board","bulletin board"}));
	set_carryover(STOP_SAVE);
	messages = 0;
	title = ([]);
	seteuid(getuid(this_object()));
}

void set_location(string l)
{
   location = l;
   ob::move(l);
}

int move(mixed dest)
{
   return MOVE_NOT_ALLOWED;
}

void set_save_file(string s)
{
   string *tmpa;
   string temp_file;
   mapping *temp_messages;

   file = save_dir + s;
   seteuid(geteuid(previous_object()));
   restore_object(file);
   if (num_messages < query_max_messages())
      return;
   tmpa = explode(file,"/");
   temp_file = attic_dir + tmpa[sizeof(tmpa) - 1] + "_" + time();
   temp_messages =
		messages[(num_messages-query_carryover())..(num_messages - 1)];
   messages = messages[0..(num_messages - query_carryover() - 1)];
   num_messages = sizeof(messages);
   save_object(temp_file);
   messages = temp_messages;
   num_messages = sizeof(messages);
   save_object(file);
}

string unique_name()
{
	int pid;

	sscanf(file_name(this_player()),"/std/user#%d",pid);
	return "/tmp/bb_" + pid;
}

int new_post(string str)
{
   if (!valid_post())
        return 0;
   if (!str)
      str = "< no title >";

   if(title[this_player()])
      write("Warning! You are already marked as posting!\n");

   if (num_messages >= query_max_messages()) {
      write("Too many messages.  You must remove some first.\n");
      return 1;
   }
   if (strlen(str) > 50) {
      write("Title is too long.\n");
      return 1;
   }
   title[this_player()] = str;

   write("Editing message.\n");
   this_player()->edit(unique_name(),"postit", this_object());
   return 1;
}

void abort()
{
   rm ((string)this_player()->query_edit_filename());
   map_delete(title,this_player());
}

varargs int postit(string mtitle)
{
	string filename;
	mapping msg;
   
	filename = (string)this_player()->query_edit_filename();
	msg = allocate_mapping(4);
	msg["title"] = mtitle ? mtitle : title[this_player()];
	msg["poster"] = (string)this_player()->query("cap_name");
	msg["time"] = time();
	msg["body"] = read_file(filename);
	rm(filename);
	this_player()->set_edit_filename("");
	if (!msg["body"])
		return 0;
	if (!messages)
		messages = ({msg});
	else
		messages += ({ msg });
	write("Posted.\n");
	num_messages++;
	map_delete(title,this_player());
	save_object(file);
	return 1;
}


int read(string str)
{
    int i;
    string tmp;
   
   if (str == 0 || 
         (sscanf(str, "%d", i) != 1 && sscanf(str, "note %d", i) != 1) || 
      (i < 0)) {
      notify_fail("Which message do you want to read?\n");
      return 0;
   }
   if (i >= num_messages) {
      write("Not that many messages.\n");
      return 1;
   }
   if (this_object()->valid_note_read(i)) {
     tmp = "Note "+i+" [" + messages[i]["poster"] + "] ";
    tmp += messages[i]["title"];
     tmp += " (" + get_time(messages[i]["time"]) + ")";
     if (messages[i]["doer"])
        tmp += " <" + messages[i]["doer"] + ">\n";
      else tmp += "\n";
      write(tmp);
      this_player()->more(explode(query_info(i,"body"),"\n"));
      return 1;
   }
   else
       return 0;
}


int remove_msg(string str)
{
   string poster,tmp;
   int i,j;
   string *tmpmsg;
   
   if (str == 0 || (sscanf(str, "%d", i) != 1 && 
            sscanf(str, "note %d", i) != 1) || i < 0) {
      notify_fail("Which message do you want to remove?\n");
      return 0;
   }
   if (i >= num_messages) {
      write("Not that many messages.\n");
      return 1;
   }
   if (!valid_remove(i))
      return 0;
   messages = messages[0..(i-1)]+messages[(i+1) .. (sizeof(messages) - 1)];
   write ("Message "+i+" removed.\n");
   num_messages--;
   save_object(file);
   return 1;
}

int edit_note(string str)
{
   int i, tmp;
   string *lines;
   string fl;
   int j;
   
	if (str == 0 || 
		 (sscanf(str, "%d", i) != 1 && sscanf(str, "note %d", i) != 1) || 
	  (i < 0)) {
	  notify_fail("Which message do you want to edit?\n");
	  return 0;
	}
	if (i > num_messages) {
	  write("Not that many messages.\n");
	  return 1;
	}
	lines = explode(query_info(i,"body"),"\n");
	if (!valid_edit(i)) {
		write("You may only edit your own messages.\n");
		return 1;
	}
	fl = unique_name();
	rm(fl);
	for(j = 0; j < sizeof(lines); j++)
		write_file(fl,lines[j] + "\n");
	orig_number = i;
	write ("Editing : " + query_info(i,"title") + "\n");
	this_player()->edit(fl,"edit_stop_ed",this_object());
	return 1;
}

void edit_stop_ed()
{
   messages[orig_number]["body"] =
		read_file((string)this_player()->query_edit_filename());
   write ("Message "+orig_number+" edited.\n");
   abort();
   save_object(file);
}

int followup_note(string str)
{
    string hed, *lines;
    int i;

    if (!valid_post())
	return 0;
    if (!str ||
	(sscanf(str, "%d %s", i, hed) != 2 &&
	 sscanf(str, "note %d %s", i, hed) != 2 &&
	 sscanf(str, "%d", i) != 1 &&
	 sscanf(str, "note %d", i) != 1)
	|| (i < 0))
    {
	notify_fail("Usage: followup <num> [<title>]\n");
	return 0;
    }
    if (i >= num_messages)
    {
	write("Not that many messages.\n");
	return 1;
    }
    if (num_messages >= query_max_messages())
    {
	write("Too many messages.  You must remove some first.\n");
	return 1;
    }
    if(title[this_player()])
	write("Warning! You are already marked as posting!\n");
    if (!hed)
	hed = "Re: " + messages[i]["title"];
    if (strlen(hed) > 50)
    {
	write("Title is too long.\n");
	return 1;
    }
    title[this_player()] = hed;
    write("Following up to note " + i + " as '" + hed + "'\n");
    lines = explode(messages[i]["body"],"\n");
    hed = unique_name();
    rm(hed);
    write_file(hed, "On " + get_time(messages[i]["time"]) + ", " + 
		    messages[i]["poster"] + " writes:\n" );
    for(i = 0; i < sizeof(lines); i++)
	write_file(hed,"> " + lines[i] + "\n");
    this_player()->edit(hed,"postit",this_object());
    return 1;
}

int save_note(string str)
{
    int i;
    string fil, lines;

    if (!str || sscanf(str, "%d %s", i, fil) != 2 || (i < 0))
    {
	notify_fail("Usage: savenote <num> <file>\n");
	return 0;
    }
    if (i >= num_messages)
    {
	write("Not that many messages.\n");
	return 1;
    }
    fil = resolv_path("cwd", fil);
    if (!master()->valid_write(fil,(string)this_player()->query("name"),
	"savenote"))
    {
	notify_fail(fil + ": Permission denied.\n");
	return 0;
    }
    write_file(fil, "Title: " + messages[i]["title"] + "\n"); 
    write_file(fil, "From:  " + messages[i]["poster"] + "\n"); 
    write_file(fil, "Date:  " + get_time(messages[i]["time"]) + "\n"); 
    lines = messages[i]["doer"];
    if (lines)
	write_file(fil, "Doer:  " + lines + "\n");
    lines = messages[i]["body"];
    write_file(fil, "\n" + lines + "\n");
    write("Note " + i + " written to " + fil + "\n");
    return 1;
}

clean_up()
{
	return;
}
